@@ -15,7 +15,7 @@
 |*
 |*    Copyright 1994, 2017 Eduardo Santiago
 |*    SPDX-License-Identifier: MIT
-\*/
+ */
 
 #include <X11/Intrinsic.h>
 #include <X11/StringDefs.h>
@@ -83,21 +83,21 @@ typedef union wait      waitType;
 **                                prototypes                                 **
 \*****************************************************************************/
 char		*doScan();
-void		Popdown(),Popup();
+void		Popdown(), Popup();
 void		Usage();
 extern char	*getlogin();
 
-void	Shrink(Widget, caddr_t, XEvent*, Boolean*);
-void	handler(XtPointer,XtIntervalId*);
-void	initStaticData(int*,int*,int*);
+void	Shrink(Widget, XtPointer, XEvent*, Boolean*);
+void	handler(XtPointer, XtIntervalId*);
+void	initStaticData(int*, int*, int*);
 void	Exit(Widget, XEvent*, String*, Cardinal*);
 void	Mailer(Widget, XEvent*, String*, Cardinal*);
 void	lbiffUnrealize(), lbiffRealize(char*);
-void	getDimensions(char*,Dimension*,Dimension*);
+void	getDimensions(char*, Dimension*, Dimension*);
 void	toggle_key_led(int);
 void	init_randr();
-void	ErrExit(Boolean,char*);
-Bool	CheckEvent(Display*,XEvent*,caddr_t);
+void	ErrExit(Boolean, char*);
+Bool	CheckEvent(Display*, XEvent*, XPointer);
 waitType popen_nmh(char *cmd, int bufsize, char **buf_out, size_t *size_out);
 
 /*****************************************************************************\
@@ -105,14 +105,14 @@ waitType popen_nmh(char *cmd, int bufsize, char **buf_out, size_t *size_out);
 \*****************************************************************************/
 extern int errno;
 
-Widget	topLevel,textBox;		/* my widgets			*/
+Widget	topLevel, textBox;		/* my widgets			*/
 XtAppContext app_context;		/* application context		*/
 Boolean	visible;			/* is window visible?		*/
 Boolean hasdata;			/* Something is to be displayed */
 char	*default_file;			/* default filename		*/
 char	*progname;			/* my program name		*/
-long	acknowledge_time = 0;		/* time window was acknowledged	*/
-long	popup_time = 0;			/* time window was popped up	*/
+struct timeval acknowledge_time = {0};	/* time window was acknowledged	*/
+struct timeval popup_time = {0};	/* time window was popped up	*/
 
 static Atom wm_delete_window;		/* for handling WM_DELETE	*/
 
@@ -122,14 +122,14 @@ typedef struct {
     char	*checkCmd;		/* command to run for check     */
     char	*cmd;			/* command to run for output	*/
     char	*mailerCmd;		/* command to read mail		*/
-    int		update;			/* update interval, in seconds	*/
-    int		fade;			/* popdown interval, in seconds */
+    float	update;			/* update interval, in seconds	*/
+    float	fade;			/* popdown interval, in seconds */
     int		columns;		/* number of columns across	*/
     int		rows;			/* max# of lines in display	*/
     int		volume;			/* bell volume, 0-100 percent	*/
     Boolean	bottom;			/* Put window at window bottom  */
     Boolean	resetSaver;		/* reset screensaver on popup   */
-    long	refresh;		/* seconds before reposting msg	*/
+    float	refresh;		/* seconds before reposting msg	*/
     int		led;			/* led number to light up	*/
     Boolean	ledPopdown;		/* turn off LED on popdown?	*/
     char	*sound;			/* Sound file to use		*/
@@ -138,6 +138,10 @@ AppData lbiff_data;
 
 #define offset(field) XtOffset(AppDataPtr, field)
 
+float default_update_secs = 15.0f;
+float default_fade_secs = 0.0f;
+float default_refresh_secs = 1800.0f;
+
 static XtResource xlbiff_resources[] = {
     {"debug", "Debug", XtRBoolean, sizeof(Boolean),
      offset(debug), XtRImmediate, False},
@@ -149,10 +153,10 @@ static XtResource xlbiff_resources[] = {
      offset(cmd), XtRString, "scan -file %s -width %d 2>&1"},
     {"mailerCommand", "MailerCommand", XtRString, sizeof(String),
      offset(mailerCmd), XtRString, NULL },
-    {"update", "Interval", XtRInt, sizeof(int),
-     offset(update), XtRImmediate, (XtPointer)15},
-    {"fade", "Fade", XtRInt, sizeof(int),
-     offset(fade), XtRImmediate, (XtPointer)0},
+    {"update", "Interval", XtRFloat, sizeof(float),
+     offset(update), XtRFloat, &default_update_secs},
+    {"fade", "Fade", XtRFloat, sizeof(float),
+     offset(fade), XtRFloat, &default_fade_secs},
     {"columns", "Columns", XtRInt, sizeof(int),
      offset(columns), XtRImmediate, (XtPointer)80},
     {"rows", "Rows", XtRInt, sizeof(int),
@@ -165,8 +169,8 @@ static XtResource xlbiff_resources[] = {
      offset(bottom), XtRImmediate, False},
     {"resetSaver", "ResetSaver", XtRBoolean, sizeof(Boolean),
      offset(resetSaver), XtRImmediate, False},
-    {"refresh", "Refresh", XtRInt, sizeof(int),
-     offset(refresh), XtRImmediate, (XtPointer)1800},
+    {"refresh", "Refresh", XtRFloat, sizeof(float),
+     offset(refresh), XtRFloat, &default_refresh_secs},
     {"led", "Led", XtRInt, sizeof(int),
      offset(led), XtRImmediate, (XtPointer)0},
     {"ledPopdown", "LedPopdown", XtRBoolean, sizeof(Boolean),
@@ -174,25 +178,25 @@ static XtResource xlbiff_resources[] = {
 };
 
 static XrmOptionDescRec optionDescList[] = {
-    {"-bottom",      ".bottom",      XrmoptionNoArg,	(caddr_t) "true"},
-    {"+bottom",      ".bottom",      XrmoptionNoArg,	(caddr_t) "false"},
-    {"-debug",       ".debug",	      XrmoptionNoArg,	(caddr_t) "true"},
-    {"-file",	      ".file",        XrmoptionSepArg,	(caddr_t) NULL},
-    {"-rows",        ".rows",	      XrmoptionSepArg,	(caddr_t) NULL},
-    {"-columns",     ".columns",     XrmoptionSepArg,	(caddr_t) NULL},
-    {"-update",      ".update",      XrmoptionSepArg,	(caddr_t) NULL},
-    {"-fade",	      ".fade",        XrmoptionSepArg,	(caddr_t) NULL},
-    {"-volume",      ".volume",      XrmoptionSepArg,	(caddr_t) NULL},
-    {"-resetSaver",  ".resetSaver",  XrmoptionNoArg,	(caddr_t) "true"},
-    {"+resetSaver",  ".resetSaver",  XrmoptionNoArg,	(caddr_t) "false"},
-    {"-refresh",     ".refresh",     XrmoptionSepArg,	(caddr_t) NULL},
-    {"-led",         ".led",         XrmoptionSepArg,	(caddr_t) NULL},
-    {"-ledPopdown",  ".ledPopdown",  XrmoptionNoArg,	(caddr_t) "true"},
-    {"+ledPopdown",  ".ledPopdown",  XrmoptionNoArg,	(caddr_t) "false"},
-    {"-sound",	      ".sound",       XrmoptionSepArg,  (caddr_t) NULL},
-    {"-scanCommand", ".scanCommand", XrmoptionSepArg,	(caddr_t) NULL},
-    {"-mailerCommand",".mailerCommand",XrmoptionSepArg,(caddr_t) NULL},
-    {"-checkCommand",".checkCommand",XrmoptionSepArg,  (caddr_t) NULL}
+    {"-bottom",      ".bottom",      XrmoptionNoArg,	(XtPointer) "true"},
+    {"+bottom",      ".bottom",      XrmoptionNoArg,	(XtPointer) "false"},
+    {"-debug",       ".debug",	      XrmoptionNoArg,	(XtPointer) "true"},
+    {"-file",	      ".file",        XrmoptionSepArg,	(XtPointer) NULL},
+    {"-rows",        ".rows",	      XrmoptionSepArg,	(XtPointer) NULL},
+    {"-columns",     ".columns",     XrmoptionSepArg,	(XtPointer) NULL},
+    {"-update",      ".update",      XrmoptionSepArg,	(XtPointer) NULL},
+    {"-fade",	      ".fade",        XrmoptionSepArg,	(XtPointer) NULL},
+    {"-volume",      ".volume",      XrmoptionSepArg,	(XtPointer) NULL},
+    {"-resetSaver",  ".resetSaver",  XrmoptionNoArg,	(XtPointer) "true"},
+    {"+resetSaver",  ".resetSaver",  XrmoptionNoArg,	(XtPointer) "false"},
+    {"-refresh",     ".refresh",     XrmoptionSepArg,	(XtPointer) NULL},
+    {"-led",         ".led",         XrmoptionSepArg,	(XtPointer) NULL},
+    {"-ledPopdown",  ".ledPopdown",  XrmoptionNoArg,	(XtPointer) "true"},
+    {"+ledPopdown",  ".ledPopdown",  XrmoptionNoArg,	(XtPointer) "false"},
+    {"-sound",	      ".sound",       XrmoptionSepArg,  (XtPointer) NULL},
+    {"-scanCommand", ".scanCommand", XrmoptionSepArg,	(XtPointer) NULL},
+    {"-mailerCommand",".mailerCommand",XrmoptionSepArg,(XtPointer) NULL},
+    {"-checkCommand",".checkCommand",XrmoptionSepArg,  (XtPointer) NULL}
 };
 
 static char *fallback_resources[] = {
@@ -202,9 +206,9 @@ static char *fallback_resources[] = {
 };
 
 static XtActionsRec lbiff_actions[] = {
-    {"exit",Exit},
-    {"mailer",Mailer},
-    {"popdown",Popdown}
+    {"exit", Exit},
+    {"mailer", Mailer},
+    {"popdown", Popdown}
 };
 
 
@@ -215,12 +219,10 @@ static XtActionsRec lbiff_actions[] = {
 /**********\
 |*  main  *|
 \**********/
-int
-main(int argc, char *argv[])
-{
+int main(int argc, char *argv[]) {
     progname = argv[0];
 
-    XtSetLanguageProc (NULL, NULL, NULL);
+    XtSetLanguageProc(NULL, NULL, NULL);
     topLevel = XtVaAppInitialize(&app_context,
                                  "XLbiff",
                                  optionDescList, XtNumber(optionDescList),
@@ -231,7 +233,7 @@ main(int argc, char *argv[])
 
     XtGetApplicationResources(topLevel, &lbiff_data,
                               xlbiff_resources, XtNumber(xlbiff_resources),
-                              (ArgList)NULL,0);
+                              (ArgList)NULL, 0);
 
 #ifndef DEBUG
     if (lbiff_data.debug)
@@ -273,12 +275,15 @@ main(int argc, char *argv[])
             username = pwd->pw_name;
         }
 
-        default_file =
-            (char *)malloc(strlen(XLBIFF_MAILPATH) + strlen(username));
+        // -2 for the "%s" removed by formatting, +1 for the NUL.
+        size_t mailpath_file_size =
+            strlen(XLBIFF_MAILPATH) - 2 + strlen(username) + 1;
+        default_file = (char *)malloc(mailpath_file_size);
         if (default_file == NULL)
             ErrExit(True, "default_file malloc()");
 
-        sprintf(default_file, XLBIFF_MAILPATH, username);
+        snprintf(default_file, mailpath_file_size, XLBIFF_MAILPATH, username);
+        default_file[mailpath_file_size - 1] = '\0';
         lbiff_data.file = default_file;
     }
     DP(("file= %s\n", lbiff_data.file));
@@ -308,7 +313,7 @@ main(int argc, char *argv[])
     XtAddCallback(textBox, XtNcallback, Popdown, textBox);
     XtAppAddActions(app_context, lbiff_actions, XtNumber(lbiff_actions));
     XtAddEventHandler(topLevel, StructureNotifyMask, False,
-                      (XtEventHandler)Shrink, (caddr_t)NULL);
+                      (XtEventHandler)Shrink, (XtPointer)NULL);
 
     XtOverrideTranslations(
         topLevel, XtParseTranslationTable("<Message>WM_PROTOCOLS: exit()"));
@@ -336,9 +341,7 @@ main(int argc, char *argv[])
 /***********\
 |*  Usage  *|  displays usage message
 \***********/
-void
-Usage()
-{
+void Usage() {
     static char *help_message[] = {
         "where options include:",
         "    -version                   display xlbiff version number",
@@ -368,13 +371,19 @@ Usage()
     exit(1);
 }
 
+// Returns true if the difference between newtime and oldtime is
+// greater than interval_seconds.
+int time_passed(struct timeval *newtime, struct timeval *oldtime,
+                float interval_seconds) {
+    float timediff_secs = newtime->tv_sec - oldtime->tv_sec +
+                          (newtime->tv_usec - oldtime->tv_usec) * 1e-6;
+    return timediff_secs > interval_seconds;
+}
 
 /**********\
 |*  Exit  *|  called via callback, exits the program
 \**********/
-void
-Exit(Widget w, XEvent *event, String *params, Cardinal *num_params)
-{
+void Exit(Widget w, XEvent *event, String *params, Cardinal *num_params) {
     DP(("++Exit()\n"));
 
     if (event->type == ClientMessage) {
@@ -403,15 +412,12 @@ Exit(Widget w, XEvent *event, String *params, Cardinal *num_params)
 |*	to execute the scanCommand.  If the result of this is non-null,
 |*	it pops up a window showing it (note that users of Berkeley
 |*	mail may have non-empty mail files with all old mail).
-\*/
-void
-checksize()
-{
+ */
+void checksize() {
     static int mailsize = 0;
     struct stat mailstat;
     int pop_window = False;
     struct timeval tp;
-    struct timezone tzp;
 
     DP(("++checksize()..."));
 
@@ -492,10 +498,10 @@ checksize()
         /*
         ** If window has been popped down, check if it's time to refresh
         */
-        if (gettimeofday(&tp, &tzp) != 0) {
+        if (gettimeofday(&tp, NULL) != 0) {
             ErrExit(True, "gettimeofday() in checksize()");
         } else {
-            if ((tp.tv_sec - acknowledge_time) > lbiff_data.refresh) {
+            if (time_passed(&tp, &acknowledge_time, lbiff_data.refresh)) {
                 DP(("reposting window, repost time reached\n"));
                 pop_window = True;
             }
@@ -506,10 +512,13 @@ checksize()
         ** if so, popdown window
         ** if fade is zero, do not pop down
         */
-        if (gettimeofday(&tp, &tzp) != 0) {
+        if (gettimeofday(&tp, NULL) != 0) {
             ErrExit(True, "gettimeofday() in checksize()");
         } else if (lbiff_data.fade > 0) {
-            if ((tp.tv_sec - popup_time) > lbiff_data.fade) lbiffUnrealize();
+            if (time_passed(&tp, &popup_time, lbiff_data.fade)) {
+                DP(("fade time (%f) reached\n", lbiff_data.fade));
+                lbiffUnrealize();
+            }
         }
     }
 
@@ -538,33 +547,33 @@ checksize()
 /************\
 |*  Mailer  *|  called via callback, starts a mailer
 \************/
-void
-Mailer(Widget w, XEvent *event, String *params, Cardinal *num_params)
-{
+void Mailer(Widget w, XEvent *event, String *params, Cardinal *num_params) {
     int system_return;
     DP(("++Mailer()\n"));
 
+    Popdown();
     if (lbiff_data.mailerCmd != NULL && lbiff_data.mailerCmd[0] != '\0') {
-        Popdown();
+        DP(("---mailerCmd = %s\n", lbiff_data.mailerCmd));
         system_return = system(lbiff_data.mailerCmd);
-        if (system_return != 0) {
+        if (system_return == 0) {
+            DP(("---mailerCmd completed successfully\n"));
+        } else {
             fprintf(stderr, "mailer command \"%s\" returned %d (%#x)\n",
                     lbiff_data.mailerCmd, system_return, system_return);
         }
-        Popup();
-        checksize();
     }
+    checksize();
+    Popup();
 }
 
 
 /*************\
 |*  handler  *|  Checks mail file and reschedules itself to do so again
 \*************/
-void
-handler(XtPointer closure, XtIntervalId *id)
-{
+void handler(XtPointer closure, XtIntervalId *id) {
     checksize();
-    XtAppAddTimeOut(app_context, lbiff_data.update * 1000, handler, NULL);
+    long int update_msecs = lbiff_data.update * 1000.0f + 0.5f;
+    XtAppAddTimeOut(app_context, update_msecs, handler, NULL);
 }
 
 
@@ -573,16 +582,14 @@ handler(XtPointer closure, XtIntervalId *id)
 |************
 |* This routine looks at the mail file and parses the contents. It
 |* does this by invoking scan(1) or some other user-defined function.
-\*/
-char *
-doScan()
-{
+ */
+char *doScan() {
     static char *cmd_buf;
     static char *buf = NULL;
-    static int	bufsize;
-    static char scan_fail_msg[] = "\n---->>>> scanCommand failed <<<<<----\n";
-    size_t	size;
-    waitType	status;
+    static int   bufsize;
+    static char  scan_fail_msg[] = "\n---->>>> scanCommand failed <<<<<----\n";
+    size_t       size;
+    waitType     status;
 
     DP(("++doScan()\n"));
 
@@ -628,9 +635,7 @@ doScan()
 /****************\
 |*  CheckEvent  *|
 \****************/
-Bool
-CheckEvent(Display *d, XEvent *e, caddr_t arg)
-{
+Bool CheckEvent(Display *d, XEvent *e, XPointer arg) {
     if (e->type == MapNotify || e->type == UnmapNotify)
         if (e->xmap.window == (Window)arg)
             return True;
@@ -649,28 +654,41 @@ static XEvent lastEvent;
 /************\
 |*  Shrink  *|  get StructureNotify events, popdown if iconified
 \************/
-void
-Shrink(Widget w, caddr_t data, XEvent *e, Boolean *b)
-{
-    DP(("++Shrink(event type %d)\n", e->type));
-    if (e->type == MapNotify || e->type == UnmapNotify) {
-        int event_seen = 0;
-        Window win = e->xmap.window;
+void Shrink(Widget w, XtPointer data, XEvent *e, Boolean *b) {
+#ifdef DEBUG
+    char *type_str;
+    switch (e->type) {
+    case UnmapNotify: type_str = "UnmapNotify"; break;
+    case MapNotify: type_str = "MapNotify"; break;
+    case ReparentNotify: type_str = "ReparentNotify"; break;
+    case ConfigureNotify: type_str = "ConfigureNotify"; break;
+    default: type_str = "event type";
+    }
+    DP(("++Shrink(%s %d)\n", type_str, e->type));
+#endif
+    if (e->type != MapNotify && e->type != UnmapNotify) {
+        return;
+    }
+    int event_seen = 0;
+    Window win = e->xmap.window;
 
-        memcpy((char *)&lastEvent, (char *)e, sizeof(XEvent));
+    memcpy((char *)&lastEvent, (char *)e, sizeof(XEvent));
 
-        XSync(XtDisplay(w), False);
+    XSync(XtDisplay(w), False);
 
-        while(XCheckIfEvent(XtDisplay(w),&lastEvent,CheckEvent,(caddr_t)win))
-            event_seen = 1;
+    while (XCheckIfEvent(XtDisplay(w), &lastEvent, CheckEvent,
+                         (XPointer)win)) {
+        event_seen = 1;
+    }
 
-        if (!event_seen)
-            return;
+    if (!event_seen) {
+        return;
+    }
 
-        if (lastEvent.type == UnmapNotify && visible)
-            Popdown();
-        else if (lastEvent.type == MapNotify && hasdata)
-            Popup();
+    if (lastEvent.type == UnmapNotify && visible) {
+        Popdown();
+    } else if (lastEvent.type == MapNotify && hasdata) {
+        Popup();
     }
 }
 
@@ -688,52 +706,35 @@ Shrink(Widget w, caddr_t data, XEvent *e, Boolean *b)
 /*************\
 |*  Popdown  *|  kill window
 \*************/
-void
-Popdown()
-{
-    struct timeval tp;
-    struct timezone tzp;
-
+void Popdown() {
     DP(("++Popdown()\n"));
-
-    if (visible) {
-        XtPopdown(topLevel);
-        XSync(XtDisplay(topLevel), False);
-    }
-    visible = False;
-
-    /*
-    ** Remember when we were popped down so we can refresh later
-    */
-    if (gettimeofday(&tp, &tzp) != 0)
-        ErrExit(True, "gettimeofday() in lbiffUnrealize()");
-
-    acknowledge_time = tp.tv_sec;
-
-    if (lbiff_data.ledPopdown)		/* Turn off LED if so requested */
-        toggle_key_led(False);
+    lbiffUnrealize();
 }
 
 
-void
-Popup()
-{
+void Popup() {
     struct timeval tp;
-    struct timezone tzp;
-
-    DP(("++Popup()\n"));
 
+    DP(("++Popup() hasdata=%d visible=%d\n", hasdata, visible));
+    if (!hasdata || visible) {
+      return;
+    }
     /*
     ** Remember when we were popped up so we can fade later
     */
-    if (gettimeofday(&tp, &tzp) != 0)
+    if (gettimeofday(&tp, NULL) != 0)
         ErrExit(True, "gettimeofday() in Popup()");
-    popup_time = tp.tv_sec;
+    popup_time = tp;
 
-    if (hasdata && !visible) {
-        XtPopup(topLevel, XtGrabNone);
-        XSync(XtDisplay(topLevel), False);
+    if (lbiff_data.bottom) {
+        Arg args[1];
+        int n = 0;
+        XtSetArg(args[n], XtNy, -1); n++;
+        XtSetValues(topLevel, args, n);
     }
+    XtRealizeWidget(topLevel);
+    XtPopup(topLevel, XtGrabNone);
+    XSync(XtDisplay(topLevel), False);
     visible = True;
 }
 
@@ -741,14 +742,26 @@ Popup()
 /********************\
 |*  lbiffUnrealize  *|  kill window
 \********************/
-void
-lbiffUnrealize()
-{
+void lbiffUnrealize() {
     DP(("++lbiffUnrealize()\n"));
-    if (lbiff_data.bottom)
+
+    /*
+    ** Remember when we were popped down so we can refresh later
+    */
+    struct timeval tp;
+    if (gettimeofday(&tp, NULL) != 0)
+        ErrExit(True, "gettimeofday() in lbiffUnrealize()");
+
+    acknowledge_time = tp;
+
+    if (visible) {
+        DP(("calling XtUnrealizeWidget\n"));
         XtUnrealizeWidget(topLevel);
-    else
-        Popdown();
+        XSync(XtDisplay(topLevel), False);
+    }
+
+    if (lbiff_data.ledPopdown)		/* Turn off LED if so requested */
+        toggle_key_led(False);
 
     visible = False;
 }
@@ -757,9 +770,7 @@ lbiffUnrealize()
 /******************\
 |*  lbiffRealize  *|  reformat window, set the text and bring window up
 \******************/
-void
-lbiffRealize(char *s)
-{
+void lbiffRealize(char *s) {
     Arg args[4];
     int n;
     static int first_time = 1;
@@ -770,7 +781,7 @@ lbiffRealize(char *s)
     ** Set the contents of the window
     */
     n = 0;
-    XtSetArg(args[n], XtNlabel,s); n++;
+    XtSetArg(args[n], XtNlabel, s); n++;
     XtSetValues(textBox, args, n);
 
     /*
@@ -778,19 +789,15 @@ lbiffRealize(char *s)
     ** is before realize()ing it.  This is so the WM can position it
     ** properly at the bottom of the screen.
     */
-    if (lbiff_data.bottom) {
-        Dimension width, height;
+    Dimension width, height;
 
-        getDimensions(s, &width, &height);
+    getDimensions(s, &width, &height);
 
-        n = 0;
-        XtSetArg(args[n], XtNwidth, width); n++;
-        XtSetArg(args[n], XtNheight, height); n++;
-        XtSetArg(args[n], XtNy, -1); n++;
+    n = 0;
+    XtSetArg(args[n], XtNwidth, width); n++;
+    XtSetArg(args[n], XtNheight, height); n++;
+    XtSetValues(topLevel, args, n);
 
-        XtSetValues(topLevel, args, n);
-        XtRealizeWidget(topLevel);
-    }
     Popup();
 
     if (first_time) {
@@ -835,17 +842,13 @@ lbiffRealize(char *s)
 
     if (lbiff_data.resetSaver)
         XResetScreenSaver(XtDisplay(topLevel));
-
-    visible = True;
 }
 
 
 /*******************\
 |*  getDimensions  *|  get width x height of text string
 \*******************/
-void
-getDimensions(char *s, Dimension *width, Dimension *height)
-{
+void getDimensions(char *s, Dimension *width, Dimension *height) {
     Dimension tmp_width;
     int i, len = strlen(s);
     static int fontWidth, fontHeight;
@@ -887,12 +890,10 @@ getDimensions(char *s, Dimension *width, Dimension *height)
 /********************\
 |*  initStaticData  *|  initializes font size & borderWidth
 \********************/
-void
-initStaticData(int *bw, int *fontH, int *fontW)
-{
+void initStaticData(int *bw, int *fontH, int *fontW) {
     Arg args[2];
-    XFontStruct *fs;
-    int tmp;
+    XFontStruct *fs = NULL;
+    int tmp = 0;
 
     DP(("++initStaticData..."));
     XtSetArg(args[0], XtNfont, &fs);
@@ -912,9 +913,7 @@ initStaticData(int *bw, int *fontH, int *fontW)
 /********************\
 |*  toggle_key_led  *|  toggle a keyboard LED on and off
 \********************/
-void
-toggle_key_led(int flag)
-{
+void toggle_key_led(int flag) {
     XKeyboardControl keyboard;
 
     if (lbiff_data.led == 0)		/* return if no led action desired */
@@ -941,10 +940,8 @@ toggle_key_led(int flag)
 |* indicated error.  It then clears the LEDs and exits.
 |*
 |* It is the intention that someday this will bring up a popup window.
-\*/
-void
-ErrExit(Boolean errno_valid, char *s)
-{
+ */
+void ErrExit(Boolean errno_valid, char *s) {
     if (errno_valid)
         fprintf(stderr, "%s: %s: %s\n", progname, s, strerror(errno));
     else
@@ -957,9 +954,8 @@ ErrExit(Boolean errno_valid, char *s)
     exit(1);
 }
 
-waitType
-popen_simple(char *cmd, int bufsize, char **buf_out, size_t *size_out)
-{
+waitType popen_simple(char *cmd, int bufsize, char **buf_out,
+                      size_t *size_out) {
     FILE *p;
     size_t read_size;
     waitType status;
@@ -972,10 +968,12 @@ popen_simple(char *cmd, int bufsize, char **buf_out, size_t *size_out)
         ErrExit(True, "popen");
     read_size = fread(*buf_out, 1, bufsize, p);
     if (read_size == bufsize) {
+        --read_size;              // leave room for NUL we add later
         char junkbuf[100];
         while (fread(junkbuf, 1, 100, p) > 0)
             ;	/* Keep reading until no more left */
     }
+    (*buf_out)[read_size] = '\0';
 
 #ifdef INTWAITTYPE
     status = pclose(p);
@@ -996,11 +994,9 @@ popen_simple(char *cmd, int bufsize, char **buf_out, size_t *size_out)
 |*      Does nmh initialization if necessary, so nmh commands such as "scan"
 |*      work even if the user has not run install-mh.
 |*      Handling this case here means xlbiff works out of the box.
-\*/
+ */
 #define PROFILE_TEMPLATE "xlbiff-mh-profile-XXXXXX"
-waitType
-popen_nmh(char *cmd, int bufsize, char **buf_out, size_t *size_out)
-{
+waitType popen_nmh(char *cmd, int bufsize, char **buf_out, size_t *size_out) {
     waitType status;
     char *profile_name;
     char *tmpdir_name;
@@ -1039,7 +1035,11 @@ popen_nmh(char *cmd, int bufsize, char **buf_out, size_t *size_out)
     strcat(profile_name, "/");
     strcat(profile_name, PROFILE_TEMPLATE);
 
+    mode_t old_mask = umask(077);
     profile_fd = mkstemp(profile_name);
+    if (profile_fd < 0)
+        ErrExit(True, "profile_name mkstemp()");
+    umask(old_mask);
     DP(("created profile_name = %s\n", profile_name));
     profile_stream = fdopen(profile_fd, "w");
     fprintf(profile_stream, "Path: %s\nWelcome: disable\n", tmpdir_name);
@@ -1068,10 +1068,8 @@ popen_nmh(char *cmd, int bufsize, char **buf_out, size_t *size_out)
  */
 
 /* helper function to handle RANDR screen change events */
-void
-handle_screen_change(Widget w, XtPointer client_data, XEvent* event,
-                     Boolean* continue_to_dispatch)
-{
+void handle_screen_change(Widget w, XtPointer client_data, XEvent *event,
+                          Boolean *continue_to_dispatch) {
     DP(("++screen_change\n"));
     int old_screen_width = WidthOfScreen(XtScreen(w));
     int old_screen_height = HeightOfScreen(XtScreen(w));
@@ -1094,18 +1092,14 @@ handle_screen_change(Widget w, XtPointer client_data, XEvent* event,
 }
 
 /* helper function to tell Xt where to dispatch RANDR screen change events */
-Boolean
-dispatch_screen_change(XEvent *event)
-{
+Boolean dispatch_screen_change(XEvent *event) {
     return XtDispatchEventToWidget(topLevel, event);
 }
 
 /*
  * Initialize RANDR extension to track when the bottom of the screen changes.
  */
-void
-init_randr()
-{
+void init_randr() {
     int event1, error1;
     if (!XRRQueryExtension(XtDisplay(topLevel), &event1, &error1)) {
         DP(("XRRQueryExtension returned False\n"));
